const isProd = process.env.NODE_ENV === 'production';

const express = require('express'),
    path = require('path'),
    router = express.Router(),
    fileDirectoryName = 'files',
    fileDirectory = isProd ? path.join(__dirname, '../../../../' + fileDirectoryName) : path.join(__dirname, '../' + fileDirectoryName),
    configFile = path.join(__dirname, 'config.json'),
    fs = require('fs'),
    iconv = require('iconv-lite'),
    chokidar = require('chokidar'),
    axios = require('axios');

const {networkInterfaces} = require('os');
const log = require('simple-node-logger').createSimpleFileLogger(path.join(__dirname, 'log.txt'));
const { readdirSync } = require('fs')


//GET home page.
router.get('/', function (req, res, next) {
    const folders = getSplitFolders();
    res.render('reports', {title: 'Звіти', folders});
});

router.get('/operations', function (req, res) {
    const folders = getSplitFolders();

    res.render('operations', {title: 'Службові операції', folders});
});
router.get('/config', function (req, res) {
    res.render('config', {
        title: 'Конфігурація',
        ip: getIp(),
        splitByCashdesk: getSplitConfig(),
        localIp: getLocalIp(),
        isPrinterInstalled: isPrinterInstalled,
        projectRootPath: fileDirectory,
        isProd: isProd,
        loging: fs.readFileSync(path.join(__dirname, 'log.txt'))
    })
});
router.post('/config/save', function (req, res) {
    const ip = req.body.ip;
    const splitByCashdesk = req.body.splitByCashdesk;
    const projectRootPath = req.body.projectRootPath;
    const file = getConfigFile();
    file['ip'] = ip;
    file['splitByCashdesk'] = splitByCashdesk;
    file['projectRootPath'] = projectRootPath;
    let data = JSON.stringify(file);

    saveConfigFile(data, res);
    res.redirect('back');
    res.end()
});
router.post('/operations/income', function (req, res) {
    const txt = `i;${req.body.income};`
    saveFile(txt, req, res);
});
router.post('/operations/refund', function (req, res) {
    const txt = `i;${req.body.refund * -1};`
    saveFile(txt, req, res);
});


router.post('/operations/income/:printerId', function (req, res) {
    req.body.printerId = req.params.printerId

    const txt = `i;${req.body.income};`
    saveFile(txt, req, res);
});
router.post('/operations/refund/:printerId', function (req, res) {
    req.body.printerId = req.params.printerId

    const txt = `i;${req.body.refund * -1};`
    saveFile(txt, req, res);
});

router.post('/reportNull', function (req, res) {
    saveFile('N;', req, res);
})

router.post('/reportX', function (req, res) {
    saveFile('X;', req, res);
})
router.post('/reportZ', function (req, res) {
    saveFile('Z;', req, res);
})

router.post('/reportNull/:printerId', function (req, res) {
    req.body.printerId = req.params.printerId
    saveFile('N;', req, res);
})

router.post('/reportX/:printerId', function (req, res) {
    req.body.printerId = req.params.printerId
    saveFile('X;', req, res);
})
router.post('/reportZ/:printerId', function (req, res) {
    req.body.printerId = req.params.printerId
    saveFile('Z;', req, res);
})
router.post('/print/', function (req, res) {
    let text = req.body;
    if (req.body.message) {
        text = req.body.message
    }
    saveFile(text, req, res);
    var ip = req.headers['x-forwarded-for'] || req.connection.remoteAddress;
    log.info(` | ${ip} | ${text}`);
});


const getDirectories = source =>
    readdirSync(source, { withFileTypes: true })
        .filter(dirent => dirent.isDirectory())
        .map(dirent => dirent.name)

function getIp() {
    let IP = null;
    try {
        const f = fs.readFileSync(configFile)
        IP = JSON.parse(f).ip;
    } catch (e) {
        console.log(e)
    }
    return IP;
}
function getSplitFolders() {
    let folders = [];
    if(getSplitConfig()) {
        const dir = path.resolve(fileDirectory);
        folders = getDirectories(dir)
    }
    return folders
}
function getSplitConfig() {
    let value = false;
    try {
        const f = fs.readFileSync(configFile)
        value = JSON.parse(f).splitByCashdesk == 'true';
    } catch (e) {
        console.log(e)
    }
    return value;
}

function getConfigFile() {
    let file = null;
    try {
        const f = fs.readFileSync(configFile)
        file = JSON.parse(f);
    } catch (e) {
        console.log(e)
    }
    return file;
}

function getLocalIp() {
    const nets = networkInterfaces();
    const results = []; // or just '{}', an empty object
    for (const name of Object.keys(nets)) {
        for (const net of nets[name]) {
            if (net.family === 'IPv4' && !net.internal) {
                results.push(net.address);
            }
        }
    }
    return results[0]
}

function isPrinterInstalled() {
    return !getIp() || getIp() === getLocalIp()
}

function saveFile(txt, req, res) {
    const printerId = req.body.printerId || null;
    const shouldSplit = getSplitConfig();
    const date = new Date().getTime();
    const writeFile = (context) => {
        const buf = iconv.encode(context, 'win1251');
        let dir = null;
        if (shouldSplit && printerId) {
            dir = path.resolve(fileDirectory, printerId.toString());
            fs.mkdirSync(dir, {recursive: true});
        } else {
            dir = fileDirectory;
            fs.mkdirSync(dir, {recursive: true});
        }

        fs.writeFile(`${dir}/${date}.txt`, buf, (response) => {
            if (!response) {
                //emulatePrinter(date);
                res.status(200).json({message: 'Print success'}).end();
            } else {
                res.status(400).json({message: 'No access to ' + fileDirectory}).end();
            }
        });
    }

    if (isPrinterInstalled()) {
        writeFile(txt);
    } else {
        const CancelToken = axios.CancelToken;
        let cancel;

        axios
            .post('http://' + getIp() + ':3000/print/',
                {message: txt},
                {
                    cancelToken: new CancelToken(function executor(c) {
                        cancel = c;
                    }),
                })
            .then((response) => {
                cancel = null
                res.status(response.status).json(response.message).end();
            })
            .catch((error) => {
                if (axios.isCancel(error)) {
                    res.status(400).json({message: 'Redirect error'}).end();
                }
            });
        setTimeout(() => {
            if (cancel) {
                cancel;
                res.status(400).json({message: 'Timeout'}).end();
            }
        }, 12000)
    }
}

function saveConfigFile(txt, res) {
    // const date = 'mycustomprintfile'
    const writeFile = (context) => {
        const buf = context;

        fs.writeFile(configFile, buf, (response) => {
            if (!response) {
                res.send();
            } else {
                res.send(response);
            }
        });
    }
    writeFile(txt);
}

function watchPrintSuccess(filePath, callBack) {
    let isWatching = true;
    var watcher = chokidar.watch([`${filePath}`],
        {
            persistent: true
        });
    watcher.on('add',
        path => {
            isWatching = false;
            const isSuccess = path.includes(fileDirectory);
            watcher.unwatch(file);
            watcher.close();
            return callBack(isSuccess)
        });

    setTimeout(function () {
        if (isWatching) {
            watcher.unwatch(file);
            watcher.close();
            return callBack(false)
        }
    }, 12000);
}

//
// function emulatePrinter(date) {
//     fs.readFile(`${fileDirectory}/${date}.txt`, (err, data) => {
//         if (err) throw err;
//         const buf = iconv.encode(data, 'win1251');
//         setTimeout(() => {
//             const dir = successDirectory // Math.floor(Math.random() * 2) ? successDirectory : errorDirectory
//             fs.writeFile(`${dir}/${date}.txt`, buf, (response) => {
//
//             });
//         }, 1000)
//     });
// }

module.exports = router;
